TITAN_REAGENTTRACKER_ID = "ReagentTracker"

local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local _, class = UnitClass("player")

-- database of buff spells and reagents
local buffs = {
	DEATHKNIGHT = {
		{	-- Raise Dead
			reagentIDs = {
				37201,
			},
			spellIDs = {
				46584,
			},
		},
	},
	DRUID = {
		{	-- Gift of the Wild
			reagentIDs = {
				[1] = 17021,	-- Wild Berries
				[2] = 17026,	-- Wild Thornroot
				[3] = 22148,	-- Wild Quillvine
				[4] = 44605,	-- Wild Spineleaf
			},
			spellIDs = {
				21849,
			},
		},
		{	-- Rebirth
			reagentIDs = {
				[1] = 17034,	-- Maple Seed
				[2] = 17035,	-- Stranglethorn Seed
				[3] = 17036,	-- Ashwood Seed
				[4] = 17037,	-- Hornbeam Seed
				[5] = 17038,	-- Ironwood Seed
				[6] = 22147,	-- Flintweed Seed
				[7] = 44614,	-- Starleaf Seed
			},
			spellIDs = {
				20484,
			},
		},
	},
	MAGE = {
		{	-- Arcane Brilliance
			reagentIDs = {
				17020,
			},
			spellIDs = {
				23028,
			},
		},
		{	-- Teleports
			reagentIDs = {
				17031,
			},
			spellIDs = {
				53140,	-- Dalaran
				33690,	-- Shattrath (Alliance)
				35715,	-- Shattrath (Horde)
				3561,	-- Stormwind
				3562,	-- Ironforge
				3565,	-- Darnassus
				32271,	-- Exodar
				3567,	-- Orgrimmar
				3563,	-- Undercity
				3566,	-- Thunder Bluff
				32272,	-- Silvermoon
			},
		},
		{	-- Portals
			reagentIDs = {
				17032,
			},
			spellIDs = {
				53142,	-- Dalaran
				33691,	-- Shattrath
				35717,	-- Shattrath
				10059,	-- Stormwind
				11416,	-- Ironforge
				11419,	-- Darnassus
				32266,	-- Exodar
				11417,	-- Orgrimmar
				11418,	-- Undercity
				11420,	-- Thunder Bluff
				32267,	-- Silvermoon
			},
		},
		{	-- Slow fall
			reagentIDs = {
				17056,
			},
			spellIDs = {
				130,
			},
		},
	},
	PALADIN = {
		{	-- Greater Blessings
			reagentIDs = {
				21177,
			},
			spellIDs = {
				25782,	-- Might
				25894,	-- Wisdom
			},
		},
		{	-- Divine Intervention
			reagentIDs = {
				17033,
			},
			spellIDs = {
				19752,
			},
		},
	},
	PRIEST = {
		{	-- Prayer of Fortitude
			reagentIDs = {
				[1] = 17028,	-- Holy Candle
				[2] = 17029,	-- Sacred Candle
				[3] = 17029,	--   -- "" --
				[4] = 44615,	-- Devout Candle
			},
			spellIDs = {
				21562,
			},
		},
		{	-- Prayer of Shadow Protection
			reagentIDs = {
				[1] = 17029,	-- Sacred Candle
				[2] = 17029,	--   -- "" --
				[3] = 44615,	-- Devout Candle
			},
			spellIDs = {
				27683,
			},
		},
		{
			-- Levitate
			reagentIDs = {
				17056,
			},
			spellIDs = {
				1706,
			},
		}
	},
	SHAMAN = {
		{	-- Reincarnation
			reagentIDs = {
				17030,
			},
			spellIDs = {
				20608,
			},
		},
		{	-- Water breathing
			reagentIDs = {
				17057,
			},
			spellIDs = {
				131,
			},
		},
		{	-- Water walking
			reagentIDs = {
				17058,
			},
			spellIDs = {
				546,
			},
		},
	},
	WARLOCK = {
		{	-- Soul Shard
			reagentIDs = {
				6265,
			},
			spellIDs = {
				697,	-- Summon Voidwalker
				6201,	-- Create Healthstone
			},
		},
		{	-- Inferno
			reagentIDs = {
				1122,
			},
			spellIDs = {
				5565,
			},
		},
		{	-- Ritual of Doom
			reagentIDs = {
				16583,
			},
			spellIDs = {
				18540,
			},
		},
	}
}

-- store spells that are known here later
local possessed = {}


function TitanPanelReagentTrackerButton_OnLoad(self)
	self.registry = {
		id = TITAN_REAGENTTRACKER_ID,
		version = GetAddOnMetadata("TitanReagentTracker", "Version"),
		menuText = "Reagent Tracker",
		tooltipTitle = "Reagent Tracker Info", 
		tooltipTextFunction = "TitanPanelReagentTrackerButton_GetTooltipText",
		savedVariables = {
			TrackReagent1 = false,
			TrackReagent2 = false,
			TrackReagent3 = false,
			TrackReagent4 = false,
			ShowSpellIcons = false,
		}
	}
	
	-- don't continue if no buffs are associated with the class
	if buffs[class] then
		self:RegisterEvent("PLAYER_LOGIN")
		self:RegisterEvent("LEARNED_SPELL_IN_TAB")
		self:RegisterEvent("BAG_UPDATE")
	end
end


local queryTooltip = CreateFrame("GameTooltip", "TitanReagentTrackerTooltip", nil, "GameTooltipTemplate")
queryTooltip:SetOwner(UIParent, "ANCHOR_NONE")
queryTooltip:SetScript("OnTooltipSetItem", function(self)
	if TitanReagentTrackerTooltipTextLeft1:GetText() ~= RETRIEVING_ITEM_INFO then
		TitanPanelReagentTracker_RefreshReagents()
		TitanPanelReagentTracker_UpdateButton()
		TitanPanelButton_UpdateTooltip(TitanPanelReagentTrackerButton)
	end
end)


function TitanPanelReagentTrackerButton_OnEvent(self, event, ...)
	if event ~= "BAG_UPDATE" then
		TitanPanelReagentTracker_RefreshReagents()
	end
	
	TitanPanelReagentTracker_UpdateButton()
	TitanPanelButton_UpdateTooltip(self)
end


function TitanPanelReagentTracker_RefreshReagents()
	wipe(possessed)
	for index, buff in ipairs(buffs[class]) do
		for i = 1, #buff.spellIDs do
			local name, rank, icon = GetSpellInfo((GetSpellInfo(buff.spellIDs[i])))
			if name then
				local rank = tonumber(rank:match("(%d+)"))
				-- use the first reagent in the table if reagentIDs[rank] doesn't exist (multi spell reagent)
				local reagentID = buff.reagentIDs[rank] or buff.reagentIDs[1]
				local reagentName = GetItemInfo(reagentID)
				if not reagentName then
					queryTooltip:SetHyperlink("item:"..reagentID)
					break
				end
				local reagentIcon = GetItemIcon(reagentID)
				-- add this spell to our "known spells" table
				possessed[index] = {
					reagentName = reagentName,
					reagentIcon = reagentIcon,
					spellIcon = icon,
				}
				break
			end
		end
	end
end


function TitanPanelReagentTracker_UpdateButton()
	local tracking
	local totalWidth = 0
	local offset = 0
	local name = "TitanPanelReagentTrackerButtonReagent"

	for index, buff in pairs(possessed) do
		local button = _G[name..index]
		local nextButton = _G[name..(index + 1)]
		local nextAnchor = "LEFT"
		local nextOffset = 0
		
		-- show/hide reagent trackers
		if TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			local icon = _G[name..index.."Icon"]
			button:Show()
			-- display spell or reagent icon
			if TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons") then
				icon:SetTexture(buff.spellIcon)
			else
				icon:SetTexture(buff.reagentIcon)
			end
			
			-- current number of reagents
			button:SetText(GetItemCount(buff.reagentName))
			
			if nextButton then
				nextAnchor = "RIGHT"
				nextOffset = 6
			end
			
			button:SetWidth(icon:GetWidth() + button:GetTextWidth())
			totalWidth = totalWidth + button:GetWidth()
			
			offset = offset + 1
			tracking = true
		else
			button:Hide()
		end
		
		-- fix offset to next reagent tracker
		if nextButton then
			nextButton:SetPoint("LEFT", button, nextAnchor, nextOffset, 0)
		end
	end
	
	-- show label if no tracking is enabled
	local none = _G[name.."None"]
	if tracking then
		none:Hide()
	else
		none:Show()
		totalWidth = none:GetWidth() + 8
	end
	
	-- adjust width so other plugins are properly offset
	TitanPanelReagentTrackerButton:SetWidth(totalWidth + ((offset - 1) * 8))
end


function TitanPanelReagentTrackerButton_GetTooltipText()
	local tooltipText = " "
	
	-- reagent info in tooltip
	for index, buff in ipairs(possessed) do
		if TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			tooltipText = tooltipText.."\n"..buff.reagentName.."\t"..TitanUtils_GetHighlightText(GetItemCount(buff.reagentName))
		end
	end
	
	if #tooltipText > 1 then
		return tooltipText
	else
		return " \nNo reagents tracked."
	end
end


function TitanPanelRightClickMenu_PrepareReagentTrackerMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_REAGENTTRACKER_ID].menuText)
	
	local info = {}
	
	-- add meny entry for each possessed spell
	for index, buff in pairs(possessed) do
		info.text = "Track "..buff.reagentName
		info.value = "TrackReagent"..index
		info.func = TitanPanelReagentTracker_ToggleVar
		info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index)
		info.keepShownOnClick = 1
		UIDropDownMenu_AddButton(info)
	end
	
	TitanPanelRightClickMenu_AddSpacer()
	
	info.text = "Show spell icons"
	info.value = "ShowSpellIcons"
	info.func = TitanPanelReagentTracker_ToggleVar
	info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons")
	info.keepShownOnClick = 1
	UIDropDownMenu_AddButton(info)
	
	TitanPanelRightClickMenu_AddSpacer()
	
	TitanPanelRightClickMenu_AddCommand(L["TITAN_PANEL_MENU_HIDE"], TITAN_REAGENTTRACKER_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end


function TitanPanelReagentTracker_ToggleVar(self)
	TitanToggleVar(TITAN_REAGENTTRACKER_ID, self.value)
	TitanPanelReagentTracker_UpdateButton()
end